@@ -2,7 +2,11 @@ from typing import List, Union, Any
 
 from pydantic import Field
 
-from rekuest.api.schema import AssignationFragment, ReserveParamsInput
+from rekuest.api.schema import (
+    AssignationFragment,
+    ReserveParamsInput,
+    ReserveBindsInput,
+)
 from koil.composition import KoiledModel
 import asyncio
 
@@ -40,10 +44,12 @@ class BasePostman(KoiledModel):
 
     async def areserve(
         self,
-        node: str,
+        node: str = None,
+        hash: str = None,
         params: ReserveParamsInput = None,
         provision: str = None,
         reference: str = "default",
+        binds: ReserveBindsInput = None,
     ) -> asyncio.Queue:
         ...
 
