@@ -1,6 +1,5 @@
 from typing import Dict
 from pydantic import Field
-from rekuest.agents.stateful import StatefulAgent
 from rekuest.api.schema import TemplateFragment
 from rekuest.postmans.graphql import GraphQLPostman
 from rekuest.rath import RekuestRath
@@ -12,12 +11,14 @@ from rekuest.structures.registry import (
 from rekuest.definition.registry import (
     DefinitionRegistry,
     get_current_definition_registry,
+    get_default_definition_registry,
 )
 from rekuest.agents.base import BaseAgent
 from rekuest.postmans.base import BasePostman
 from koil import unkoil
 from koil.composition import Composition
 from koil.decorators import koilable
+from rekuest.register import register
 
 
 @koilable(fieldname="koil", add_connectors=True)
@@ -26,10 +27,7 @@ class Rekuest(Composition):
     structure_registry: StructureRegistry = Field(
         default_factory=get_default_structure_registry
     )
-    definition_registry: DefinitionRegistry = Field(
-        default_factory=get_current_definition_registry
-    )
-    agent: BaseAgent = Field(default_factory=StatefulAgent)
+    agent: BaseAgent = Field(default_factory=BaseAgent)
     postman: BasePostman = Field(default_factory=GraphQLPostman)
 
     registered_templates: Dict[str, TemplateFragment] = Field(default_factory=dict)
@@ -38,19 +36,17 @@ class Rekuest(Composition):
         """
         Register a new function
         """
-        structure_registry = kwargs.get("structure_registry", self.structure_registry)
+        structure_registry = kwargs.pop("structure_registry", self.structure_registry)
+        definition_registry = kwargs.pop(
+            "definition_registry", self.agent.definition_registry
+        )
 
-        def real_decorator(function_or_actor):
-            self.definition_registry.register(
-                function_or_actor,
-                *args,
-                structure_registry=structure_registry,
-                **kwargs,
-            )
-
-            return function_or_actor
-
-        return real_decorator
+        return register(
+            *args,
+            definition_registry=definition_registry,
+            structure_registry=structure_registry,
+            **kwargs,
+        )
 
     def run(self, *args, **kwargs) -> None:
         """
