@@ -17,7 +17,7 @@ from rekuest.structures.errors import (
     StructureExpandingError,
 )
 from rekuest.definition.validate import auto_validate
-
+from .predication import predicate_port
 
 async def aexpand_arg(
     port: Union[PortFragment, ChildPortFragment],
@@ -33,21 +33,21 @@ async def aexpand_arg(
         Any: Expanded value
 
     """
+    if value is None:
+        value = port.default
+
     if value is None:
         if port.nullable:
             return None
         else:
-            if port.default is not None:
-                value = port.default
-            else:
-                raise ExpandingError(
-                    f"{port.key} is not nullable (optional) but your provided None"
-                )
+            raise ExpandingError(
+                f"{port.key} is not nullable (optional) but received None"
+            )
 
     if not isinstance(value, (str, int, float, dict, list)):
         raise ExpandingError(
             f"Can't expand {value} of type {type(value)} to {port.kind}. We only accept"
-            " strings, ints and floats (json serializable)"
+            " strings, ints and floats (json serializable) and null values"
         ) from None
 
     if port.kind == PortKind.DICT:
@@ -61,11 +61,25 @@ async def aexpand_arg(
             key: await aexpand_arg(port.child, value, structure_registry)
             for key, value in value.items()
         }
+    
+    if port.kind == PortKind.UNION:
+        if not isinstance(value, dict):
+            raise ExpandingError(
+                f"Can't expand {value} of type {type(value)} to {port.kind}. We only"
+                " accept dicts in unions"
+            ) 
+        assert "use" in value, "No use in vaalue"
+        index = value["use"]
+        true_value = value["value"]
+        return await aexpand_arg(port.variants[index], true_value, structure_registry=structure_registry)
+                
+
+
 
     if port.kind == PortKind.LIST:
         if not isinstance(value, list):
             raise ExpandingError(
-                f"Can't expand {value} of type {type(value)} to {port.kind}. We only"
+                f"Can't expand {value} of type {type(value)} to {port.kind}. Only"
                 " accept lists"
             ) from None
 
@@ -74,10 +88,10 @@ async def aexpand_arg(
         )
 
     if port.kind == PortKind.INT:
-        return int(value) if value is not None else int(port.default)
+        return int(value)
 
     if port.kind == PortKind.FLOAT:
-        return float(value) if value is not None else float(port.default)
+        return float(value)
 
     if port.kind == PortKind.STRUCTURE:
         try:
@@ -96,10 +110,10 @@ async def aexpand_arg(
             ) from e
 
     if port.kind == PortKind.BOOL:
-        return bool(value) if value is not None else bool(port.default)
+        return bool(value)
 
     if port.kind == PortKind.STRING:
-        return str(value) if value is not None else str(port.default)
+        return str(value)
 
     raise NotImplementedError("Should be implemented by subclass")
 
@@ -140,9 +154,11 @@ async def expand_inputs(
             }
 
         except Exception as e:
-            raise ExpandingError(f"Couldn't expand Arguments {args} ") from e
+            raise ExpandingError(f"Couldn't expand Arguments {args}") from e
     else:
-        expandend_params = {port.key: arg for port, arg in zip(node.args, args)}
+        expandend_params = {
+            port.key: arg for port, arg in zip(node.args, args) if arg is not None
+        }
 
     return expandend_params
 
@@ -169,6 +185,15 @@ async def ashrink_return(
                 raise ValueError(
                     f"{port} is not nullable (optional) but your provided None"
                 )
+            
+        if port.kind == PortKind.UNION:
+            for index, x in enumerate(port.variants):
+                if predicate_port(x, value, structure_registry):
+                    return {"use": index, "value": await ashrink_return(x, value, structure_registry)}
+                
+            raise ShrinkingError(f"Port is union butn none of the predicated for this port held true {port.variants}")
+
+
 
         if port.kind == PortKind.DICT:
             return {
@@ -189,6 +214,9 @@ async def ashrink_return(
         if port.kind == PortKind.INT:
             return int(value) if value is not None else None
 
+        if port.kind == PortKind.FLOAT:
+            return float(value) if value is not None else None
+
         if port.kind == PortKind.STRUCTURE:
             # We always convert structures returns to strings
             try:
@@ -202,10 +230,10 @@ async def ashrink_return(
             try:
                 shrink = await shrinker(value)
                 return str(shrink)
-            except Exception:
+            except Exception as e:
                 raise StructureShrinkingError(
                     f"Error shrinking {repr(value)} with Structure {port.identifier}"
-                ) from None
+                ) from e
 
         if port.kind == PortKind.BOOL:
             return bool(value) if value is not None else None
