@@ -6,6 +6,8 @@ from rekuest.structures.registry import StructureRegistry
 from rekuest.api.schema import (
     PortFragment,
     PortKind,
+    DefinitionInput,
+    DefinitionFragment,
     ChildPortFragment,
 )
 from rekuest.structures.errors import (
@@ -14,6 +16,9 @@ from rekuest.structures.errors import (
     PortExpandingError,
     StructureExpandingError,
 )
+from .predication import predicate_port
+
+
 
 
 async def ashrink_arg(
@@ -53,6 +58,16 @@ async def ashrink_arg(
 
         if port.kind == PortKind.INT:
             return int(value) if value is not None else None
+        
+        if port.kind == PortKind.UNION:
+
+            for index, x in enumerate(port.variants):
+                if predicate_port(x, value, structure_registry):
+                    return {"use": index, "value": await ashrink_arg(x, value, structure_registry)}
+                
+            raise ShrinkingError(f"Port is union butn none of the predicated for this port held true {port.variants}")
+
+
 
         if port.kind == PortKind.STRUCTURE:
             # We always convert structures returns to strings
@@ -196,6 +211,14 @@ async def aexpand_return(
                 for item in value
             ]
         )
+    
+    if port.kind == PortKind.UNION:
+        assert isinstance(value, dict), "Union value needs to be a dict"
+        assert "use" in value, "No use in vaalue"
+        index = value["use"]
+        true_value = value["value"]
+        return await aexpand_return(port.variants[index], true_value, structure_registry=structure_registry)
+                
 
     if port.kind == PortKind.INT:
         return int(value)
@@ -283,3 +306,73 @@ async def expand_outputs(
         expanded_returns = returns
 
     return tuple(expanded_returns)
+
+
+def serialize_inputs(
+    definition: Union[DefinitionFragment, DefinitionInput],
+    kwargs: Dict[str, Any],
+) -> Tuple[Any]:
+    """Shrinks args and kwargs
+
+    Shrinks the inputs according to the Node Definition
+
+    Args:
+        node (Node): The Node
+
+    Raises:
+        ShrinkingError: If args are not Shrinkable
+        ShrinkingError: If kwargs are not Shrinkable
+
+    Returns:
+        Tuple[List[Any], Dict[str, Any]]: Parsed Args as a List, Parsed Kwargs as a dict
+    """
+
+    args_list = []
+
+    # Extract to Argslist
+
+    for port in definition.args:
+        value = kwargs.pop(port.key, None)
+        if value is None and not port.nullable:
+            raise ShrinkingError(
+                f"Couldn't find value for nonnunllable port {port.key}"
+            )
+        args_list.append(value)
+
+    shrinked_args = args_list
+
+    return tuple(shrinked_args)
+
+
+def deserialize_outputs(
+    definition: Union[DefinitionFragment, DefinitionInput],
+    returns: List[Any],
+) -> Dict[str, Any]:
+    """Expands Returns
+
+    Expands the Returns according to the Node definition
+
+
+    Args:
+        node (Node): Node definition
+        returns (List[any]): The returns
+
+    Raises:
+        ExpandingError: if they are not expandable
+
+    Returns:
+        Dcit[str, Any]: The Expanded Returns
+    """
+    assert returns is not None, "Returns can't be empty"
+    if len(definition.returns) != len(returns):
+        raise ExpandingError(
+            f"Missmatch in Return Length. Node requires {len(definition.returns)} returns,"
+            f" but got {len(returns)}"
+        )
+
+    values = {}
+
+    for port, value in zip(definition.returns, returns):
+        values[port.key] = value
+
+    return values
